/*******************************************************************************
*
* Title: sangoma_isup.h
*
* Description: Sangoma_Isup library header file
*
* Written By: Konrad Hammel (konrad@sangoma.com)
*
* Date: 02-12-2010
*
*******************************************************************************/

/******************************************************************************/
#ifndef __SNG_SS7_H__
# define __SNG_SS7_H__
/******************************************************************************/

/* defines ********************************************************************/
#define S_PROC                  1                   /*this is the global PROC value to use*/
#define S_INST                  0                   /*global INSTantous value*/
#define S_REG                   0                   /*memory region value*/
#define S_POOL                  0x01                /*memory pool value*/
#define POOL_LW_TR              1
#define POOL_UP_TR              2
/******************************************************************************/

/* Limits *********************************************************************/
#define MAX_ISAPS			2
#define MAX_NSAPS			2
#define MAX_ISUP_CKTS		4096                /* max number of Circuits */
#define MAX_ISUP_INFS		12                  /* max number of SSPs */
#define MAX_MTP_ROUTES		100                 /* max number of routes configurable*/
#define MAX_MTP_LINKSETS	16                  /* max number of link sets configurable*/
#define MAX_MTP_LINKS		992                  /* max number of links configurable*/
#define MAX_RELAY_CHANNELS	16					/* max number of relay channels */
/******************************************************************************/

/* Relay Defines **************************************************************/
#define RY_PERIOD				10					/* relay timer period */
#define RY_TX_KP_ALIVE_TMR		10					/* TX TCP keep alive timer */
#define RY_RX_KP_ALIVE_TMR		50					/* RX TCP keep alive timer */
#define RY_BT_TMR				10					/* Boot Timer */
#define MAX_RELAY_CONGTHRSH		2					/* Congestion Threshold */
#define MAX_RELAY_NMBSCAN		5					/* Number of times to scan queue on read */

/* SI Defines *****************************************************************/
#define MAX_SI_VARIANTS         2                   /* max number of ISUP variants running at once */
#define MAX_SI_CIR              MAX_CC_CIRCUITS     /* same as CC */
#define MAX_SI_INTERFACES       12                  /* max number of SSPs */
#define MAX_SI_CIR_GRP          99                  /* max number of circuit groups */
#define MAX_SI_CALL_REF         4092                /* max number of active calls */
#define MAX_SI_USER_2_USER_LEN  255                 /* max size of user to user messages */
#define SI_PERIOD               10                   /* period used for timers...10,000 = 0.1s */
#define SI_LINK_SELECTION       LSI_LNKSEL_CIC      /* link is selected by cic */
/******************************************************************************/

/* SN Defines *****************************************************************/
#define MAX_SN_VARIANTS         2					/* max number of mtp3 upper saps...1 per variant */
#define MAX_LINKS_IN_LINKSET    16					/* max numbber of links in a linkset configurable*/
#define MAX_SLTM_RETRIES        2					/* number of times to send SLTM without SLTA before dropping link*/
#define SN_CB_PERIOD            10					/* 0.1ms timer resolution for CB timers in SN */
#define SN_SP_PERIOD            10					/* 0.1ms timer resolution for SP timers in SN */
#define SN_RTE_PERIOD           10					/* 0.1ms timer resolution for RTE timers in SN */
#define MAX_SN_MSG_SIZE         272
#define MAX_SN_CREDIT           128
/******************************************************************************/

/* SD Defines *****************************************************************/
#define MAX_SD_OUTSTANDING      8                  /* max number of outstanding frames */
#define MAX_SD_FRAME_LEN        273                 /* max frame length */
#define MAX_SD_SUERM            100                  /* SUERM threshold */
#define MAX_SD_AERM_EMERGENCY   10                  /* AERM threshold emergency */
#define MAX_SD_AERM_NORMAL      10                  /* AERM threshold normal */
#define MAX_SD_MSU_RETRANS      30                  /* max number of MSU to queue for retrans */
#define MAX_SD_OCTETS_RETRANS   4096                /* max number of octets to queue for retrans */
#define MAX_SD_ALIGN_ATTEMPTS   100                 /* max number of alignment attempts */
/******************************************************************************/

/* ****************************************************************************/
#define LIBSNG_SS7_TIMEOUT		100					/* cfg/sta/cntrl/sts request */ 
													/* timeout ( 50 * LIBSNG_SS7_TIMEOUT = 5 sec ) */

/* General includes/defines ***************************************************/
#ifdef __LINUX__
#include <stdint.h>
#endif

#ifdef __WINDOWS__
#endif

#include "envopt.h"        /* environment options */
#include "envdep.h"        /* environment dependent */
#include "envind.h"        /* environment independent */

#include "gen.h"           /* general layer */
#include "ssi.h"           /* system services */
#include "cm_ss7.h"

#include "lcc.h"           /* Call Contrl layer management */
#include "lsi.h"           /* ISUP layer management */
#include "lsn.h"           /* MTP3 layer management */
#include "lap.h"           /* MTP3 layer management */
#include "lsd.h"           /* MTP2 layer management */
#include "ll1.h"           /* Layer 1 layer management */
#include "lry.h"

#include "sit.h"           /* ISUP */
#include "mac.h"
#include "sdt.h"		   /* SD disconnect reasons */

#include "gen.x"           /* general layer */
#include "ssi.x"           /* system services */
#include "cm_ss7.x"

#include "lcc.x"           /* Call Control layer managament */
#include "lsi.x"           /* ISUP layer management */
#include "lsn.x"           /* MTP3 layer management */
#include "lap.x"           /* MTP3 layer management */
#include "lsd.x"           /* MTP2 layer management */
#include "ll1.x"           /* layer1 layer management*/
#include "lry.x"

#include "sit.x"           /* ISUP */
#include "mac.x"           /* Layer1 */

#include "sng_ss7_error.h"

/******************************************************************************/

/* Limits *********************************************************************/
#define MAX_CC_INTERFACE        5                   /* max number of CC interfaces */
#define MAX_SI_CIRCUITS         4096                /* max number of Circuits */
#define MAX_SI_INTERFACES       12                  /* max number of SSPs */
#define MAX_SN_INTERFACE        5                   /* max number of MTP3/ISUP type combinations...limited to 5 for now */
#define MAX_SN_ROUTES           100                 /* max number of routes configurable*/
#define MAX_SN_LINKSETS         16                  /* max number of link sets configurable*/
#define MAX_SN_LINKS            992                  /* max number of links configurable*/
#define MAX_SD_LINKS            MAX_SN_LINKS        /* max number of links configurable */
#define MAX_L1_LINKS            MAX_SD_LINKS        /* max number of sig links configurable */
#define MAX_SNGSS7_PATH			128					/* max path name */
#define MAX_SNGSS7_PC			12					/* max length of a point code */
#define MAX_MAC_SIZE 			18					/* max length of a mac address */
#define MAX_LIC_NAME			256					/* max length of names in license file */

#define MAX_MSU_SIZE			272					/* max size of a MSU frame (Q.703 section 2.3.8) */
/******************************************************************************/

/* Logging ********************************************************************/
enum
{
    SNG_LOGLEVEL_DEBUG=1,
    SNG_LOGLEVEL_WARN,
    SNG_LOGLEVEL_INFO,
    SNG_LOGLEVEL_NOTICE,
    SNG_LOGLEVEL_ERROR,
    SNG_LOGLEVEL_CRIT,
    SNG_LOGLEVEL_PUBLIC=100,
};

/* Event Call-backs structure**************************************************/
typedef struct sng_isup_event_interface
{
    struct
    {
        void (*sng_con_ind)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiConEvnt *siConEvnt);
        void (*sng_con_cfm)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiConEvnt *siConEvnt);
        void (*sng_con_sta)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiCnStEvnt *siCnStEvnt, uint8_t evntType);
        void (*sng_rel_ind)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiRelEvnt *siRelEvnt);
        void (*sng_rel_cfm)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiRelEvnt *siRelEvnt);
        void (*sng_dat_ind)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiInfoEvnt *siInfoEvnt);
        void (*sng_fac_ind)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, uint8_t evntType, SiFacEvnt *siFacEvnt);
        void (*sng_fac_cfm)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, uint8_t evntType, SiFacEvnt *siFacEvnt);
        void (*sng_sta_ind)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, uint8_t globalFlg, uint8_t evntType, SiStaEvnt *siStaEvnt);
        void (*sng_umsg_ind)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit);
        void (*sng_susp_ind)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiSuspEvnt *siSuspEvnt);
        void (*sng_resm_ind)(uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiResmEvnt *siResmEvnt);
    }cc;
    struct
    {
        void (*sng_log)(uint8_t level, char *fmt, ...);
        void (*sng_mtp1_alarm)(Pst *pst, L1Mngmt *sta);
        void (*sng_mtp2_alarm)(Pst *pst, SdMngmt *sta);
        void (*sng_mtp3_alarm)(Pst *pst, SnMngmt *sta);
        void (*sng_mtp3_li_alarm)(Pst *pst, ApMngmt *sta);
        void (*sng_isup_alarm)(Pst *pst, SiMngmt *sta);
        void (*sng_cc_alarm)(Pst *pst, CcMngmt *sta);
	void (*sng_relay_alarm)(Pst *pst, RyMngmt *sta);
    }sm;
	/*  AP call backs are only used in MTP2 API mode */
	struct
	{
		void (*sng_ap_dat_ind)(int16_t suId, void *data, int16_t len);
		void (*sng_ap_dat_cfm)(int16_t suId, void *data, int16_t len, MtpStatus status, SeqU24 credit);
		void (*sng_ap_con_cfm)(int16_t suId);
		void (*sng_ap_disc_ind)(uint16_t suId, Reason reason);
		void (*sng_ap_sta_ind)(uint16_t suId, MtpStatus status);
		void (*sng_ap_sta_cfm)(uint16_t suId, Action action, SeqU24 status);
		void (*sng_ap_flc_ind)(uint16_t suId, Action action);
	}ap;
}sng_isup_event_interface_t;
/******************************************************************************/

/* General functions **********************************************************/
extern int32_t sng_isup_init_gen(sng_isup_event_interface_t *u_sng_event_inf);
extern int32_t sng_set_procId(uint16_t procId);
extern int32_t sng_get_procId(void);
extern int32_t sng_isup_init_cc(void);
extern int32_t sng_isup_init_isup(void);
extern int32_t sng_isup_init_mtp3(void);
extern int32_t sng_isup_init_mtp3_li(void);
extern int32_t sng_isup_init_mtp2(void);
extern int32_t sng_isup_init_mtp1(void);
extern int32_t sng_isup_init_sm(void);
extern int32_t sng_isup_init_relay(void);

extern int32_t sng_isup_free_gen(void);
extern int32_t sng_isup_free_cc(void);
extern int32_t sng_isup_free_isup(void);
extern int32_t sng_isup_free_mtp3(void);
extern int32_t sng_isup_free_mtp3_li(void);
extern int32_t sng_isup_free_mtp2(void);
extern int32_t sng_isup_free_mtp1(void);
extern int32_t sng_isup_free_sm(void);
extern int32_t sng_isup_free_relay(void);

extern int32_t sng_isup_version(uint32_t *major, uint32_t *minor, uint32_t *build);
extern int32_t sng_isup_reg_info_show(void);
/******************************************************************************/

/* License Validaton **********************************************************/
extern int32_t sng_validate_license(char *license, char *signature);
/******************************************************************************/

/* Outgoing Call Functions ****************************************************/
extern int32_t sng_cc_con_request(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiConEvnt *siConEvnt, uint8_t xchgType);
extern int32_t sng_cc_con_response(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiConEvnt *siConEvnt, uint8_t xchgType);
extern int32_t sng_cc_con_status(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiCnStEvnt *siCnStEvnt, uint8_t evntType);
extern int32_t sng_cc_rel_request(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiRelEvnt *siRelEvnt);
extern int32_t sng_cc_rel_response(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiRelEvnt *siRelEvnt);
extern int32_t sng_cc_dat_request(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiInfoEvnt *siInfoEvnt);
extern int32_t sng_cc_susp_request(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiSuspEvnt *siSuspEvnt);
extern int32_t sng_cc_resm_request(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, SiResmEvnt *siResmEvnt);
extern int32_t sng_cc_sta_request(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, uint8_t globalFlg, uint8_t evntType, SiStaEvnt *siStaEvnt);
extern int32_t sng_cc_fac_request(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, uint8_t evntType, SiFacEvnt *siFacEvnt);
extern int32_t sng_cc_fac_response(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit, uint8_t evntType, SiFacEvnt *siFacEvnt);
extern int32_t sng_cc_umsg_request(uint32_t spId, uint32_t suInstId, uint32_t spInstId, uint32_t circuit);
/******************************************************************************/

/* Configuration Functions ****************************************************/
extern int32_t sng_cfg_cc(Pst *pst, CcMngmt *cfg);
extern int32_t sng_cfg_isup(Pst *pst, SiMngmt *cfg);
extern int32_t sng_cfg_mtp3(Pst *pst, SnMngmt *cfg);
extern int32_t sng_cfg_mtp3_li(Pst *pst, ApMngmt *cfg);
extern int32_t sng_cfg_mtp2(Pst *pst, SdMngmt *cfg);
extern int32_t sng_cfg_mtp1(Pst *pst, L1Mngmt *cfg);
extern int32_t sng_cfg_relay(Pst *pst, RyMngmt *cfg);
/******************************************************************************/

/* Control Functions **********************************************************/
extern int32_t sng_cntrl_cc(Pst *pst, CcMngmt *cntrl);
extern int32_t sng_cntrl_isup(Pst *pst, SiMngmt *cntrl);
extern int32_t sng_cntrl_mtp3(Pst *pst, SnMngmt *cntrl);
extern int32_t sng_cntrl_mtp3_li(Pst *pst, ApMngmt *cntrl);
extern int32_t sng_cntrl_mtp2(Pst *pst, SdMngmt *cntrl);
extern int32_t sng_cntrl_mtp1(Pst *pst, L1Mngmt *cntrl);
extern int32_t sng_cntrl_relay(Pst *pst, RyMngmt *cntrl);
/******************************************************************************/

/* Status Requests ************************************************************/
extern int32_t sng_sta_cc(Pst *pst, CcMngmt *sta, CcMngmt *cfm);
extern int32_t sng_sta_isup(Pst *pst, SiMngmt *sta, SiMngmt *cfm);
extern int32_t sng_sta_mtp3_li(Pst *pst, ApMngmt *sta, ApMngmt *cfm);
extern int32_t sng_sta_mtp3(Pst *pst, SnMngmt *sta, SnMngmt *cfm);
extern int32_t sng_sta_mtp2(Pst *pst, SdMngmt *sta, SdMngmt *cfm);
extern int32_t sng_sta_mtp1(Pst *pst, L1Mngmt *sta, L1Mngmt *cfm);
extern int32_t sng_sta_relay(Pst *pst, RyMngmt *sta, RyMngmt *cfm);
/******************************************************************************/

/* Statistics Requests ********************************************************/
extern int32_t sng_sts_cc(Pst *pst, int32_t action, CcMngmt *sts, CcMngmt *cfm);
extern int32_t sng_sts_isup(Pst *pst, int32_t action, SiMngmt *sts, SiMngmt *cfm);
extern int32_t sng_sts_mtp3_li(Pst *pst, int32_t action, ApMngmt *sts, ApMngmt *cfm);
extern int32_t sng_sts_mtp3(Pst *pst, int32_t action, SnMngmt *sts, SnMngmt *cfm);
extern int32_t sng_sts_mtp2(Pst *pst, int32_t action, SdMngmt *sts, SdMngmt *cfm);
extern int32_t sng_sts_mtp1(Pst *pst, int32_t action, L1Mngmt *sts, L1Mngmt *cfm);
extern int32_t sng_sts_relay(Pst *pst, int32_t action, RyMngmt *sts, RyMngmt *cfm);
/******************************************************************************/

/* MTP2 API Functions *************************************************************/
extern S16 sng_ss7_ap_dat_req(SuId suId, void* data, uint32_t len);
extern S16 sng_ss7_ap_sta_req(SpId spId, Action action, MtpStatus status);

/* Util Functions *************************************************************/
extern void smHdrInit(Header *hdr);
extern void smPstInit(Pst *pst);
/******************************************************************************/

/******************************************************************************/
#endif  /* __SNG_SS7_H__ */
/******************************************************************************/
